import os
import time
import datetime

import numpy as np
import pandas as pd
import matplotlib.pylab as plt
import matplotlib.dates as mdates

import seaborn as sns
# Use seaborn style defaults and set the default figure size
sns.set(rc={'figure.figsize':(11, 4)})

DATETIME = 'datetime'
PM10 = 'PM10'
PM25 = 'PM25'

data_files = {'Girton Road': 'GirtonRoad_2019-03-01-190412211355.csv',
              'Impington': 'SCambs_Impington_2019-03-01-190412211838.csv',
              'Orchard Park School': 'SCambs_Orchard_Park_School_2019-03-01-190412211559.csv'}


def remove_30_min(string: str) -> str:
    hour = int(string[:2])
    return " %02d" % (hour - 1) + ':30:00'


def load():
    """ Status: P - Provisional data, P* - Ratified by Data Provider, R - Ratified Data,V - Validated,A - As supplied"""
    dfs = {}
    for name, file in data_files.items():
        df = pd.read_csv(file, skiprows=5, skipfooter=1, engine='python')

        # create datetime column with valid hour (24 is not a valid hour)
        df['Mid Time'] = df['End Time'].apply(remove_30_min)
        df[DATETIME] = df['End Date'] + df['Mid Time']
        # Why are two different date time formats used in files from a single provider?!?
        try:
            df[DATETIME] = pd.to_datetime(df[DATETIME], format="%Y-%m-%d %H:%M:%S")
        except ValueError:
            df[DATETIME] = pd.to_datetime(df[DATETIME], format="%d/%m/%Y %H:%M:%S")

        df = df.set_index(DATETIME)
        print(df.index[0])
        df = df.drop(['Mid Time', 'End Time', 'End Date', 'Status/units', 'Status/units.1', 'Status/units.2', 'Status/units.3'], axis=1)

        dfs[name] = df

    return dfs


def plot(dfs, start_date: datetime.datetime = None, end_date: datetime.datetime = None):
    m = ['o', 's', '<']

    fig10 = plt.figure(figsize=(10, 4), dpi=200)
    ax10 = plt.gca()
    fig25 = plt.figure(figsize=(10, 4), dpi=200)
    ax25 = plt.gca()

    fmt = mdates.DateFormatter('%d %b %H:%M')
    ax10.xaxis.set_major_formatter(fmt)
    ax25.xaxis.set_major_formatter(fmt)

    ds = ''
    for ii, name in enumerate(dfs.keys()):
        df = dfs[name]

        if start_time is not None and end_date is not None:
            _fmt = '%d%b-%H%M'
            ds = '_%s-%s' % (start_date.strftime(_fmt), end_date.strftime(_fmt))
            df = df.loc[start_date:end_date]
        print(df.head())
        print(df.describe())
        print(df.dtypes)
        print(f"df.index.dtype = {df.index.dtype }")
        print(df.index)
        if PM10 in df.columns:
            dft = df[df[PM10].isnull() == False]
            ax10.plot(dft.index, dft[PM10], marker=m[ii], ms=2, lw=1, label=name)
        if PM25 in df.columns:
            ax25.plot(df.index, df[PM25], marker=m[ii], ms=2, lw=1, label=name)

    ax10.set_title('PM 10')
    ax10.legend()
    ax10.set_xlabel('Time')
    ax10.set_ylabel(r'Concentration [$\mu m^3$]')
    ax10.tick_params(axis='x', labelrotation=70)
    fig10.tight_layout()
    fig10.savefig('pm-10%s.jpg' % ds, dpi=300)

    ax10.set_ylim((0, 50))
    ax25.set_title('PM 2.5')
    ax25.legend()
    ax25.set_xlabel('Time')
    ax25.set_ylabel(r'Concentration [$\mu m^3$]')
    ax25.tick_params(axis='x', labelrotation=70)
    fig25.tight_layout()
    fig25.savefig('pm-25%s.jpg' % ds, dpi=300)


def main():
    dfs = load()
    plot(dfs)
    plot(dfs, start_date=datetime.datetime(year=2019, month=3, day=29), end_date=datetime.datetime(year=2019, month=4, day=1))
    plot(dfs, start_date=datetime.datetime(year=2019, month=4, day=6),
         end_date=datetime.datetime(year=2019, month=4, day=10))



if __name__ == '__main__':
    start_time = time.time()
    main()
    print('------ %.3f seconds ------' % (time.time() - start_time))
